library example.basic_auth.server;

import 'dart:async';
import 'dart:convert';

import 'package:jaguar/jaguar.dart';
import 'package:jaguar_session/jaguar_session.dart';

import '../../common/models/book/book.dart';
import '../../common/books.dart';

part 'server.g.dart';

CookieSession sessionManager() => new CookieSession();

/// This route group contains login and logout authentication routes
@Api()
class AuthRoutes {
  @Post(path: '/login/:username/:pwd')
  Future<Response> login(Context ctx) async {
    String username = ctx.pathParams['username'];
    String pwd = ctx.pathParams['pwd'];
    if (username == null || pwd == null) {
      throw new UnAuthorizedError();
    }

    if (username != 'jaguar' || pwd != 'awesome') {
      throw new UnAuthorizedError();
    }

    Response resp = new Response(null, statusCode: 200);
    CookieSession sesMan = sessionManager();
    //Set session information
    await sesMan.updateSession({'username': 'jaguar'});
    //Update response with session information
    return await sesMan.writeResponse(ctx.req, resp);
  }

  @Post(path: '/logout')
  Future<Response> logout(Context ctx) async {
    Response resp = new Response(null, statusCode: 200);
    CookieSession sesMan = sessionManager();
    //Clear session information on logout
    await sesMan.deleteSession();
    //Update response
    return await sesMan.writeResponse(ctx.req, resp);
  }
}

/// Collection of routes that need authorization
@Api(path: '/book')
class StudentRoutes {
  @Get(path: '/all')
  Future<Response<String>> getAllBooks(Context ctx) async {
    CookieSession sesMan = sessionManager();
    //Parse session information
    await sesMan.parseRequest(ctx.req);

    //Authorize
    String username = sesMan.getInValue('username');
    if (username is! String || username != 'jaguar') {
      throw new UnAuthorizedError();
    }

    List<Map> ret = books.values.map((Book book) => book.toMap()).toList();
    Response<String> resp = new Response<String>(JSON.encode(ret));
    return resp;
  }
}

@Api(path: '/api')
class LibraryApi {
  @IncludeApi()
  final AuthRoutes auth = new AuthRoutes();

  @IncludeApi()
  final StudentRoutes student = new StudentRoutes();
}

server() async {
  final api = new JaguarLibraryApi(new LibraryApi());

  Jaguar jaguar = new Jaguar();
  jaguar.addApi(api);

  await jaguar.serve();
}
