library example.jwt_cookie.server;

import 'dart:async';

import 'package:jaguar/jaguar.dart';
import 'package:jaguar_auth/jaguar_auth.dart';
import 'package:jaguar_session/jaguar_session.dart';
import 'package:jaguar_jwt/jaguar_jwt.dart';

import '../../common/models/book/book.dart';
import '../../common/user.dart';
import '../../common/books.dart';

part 'server.g.dart';

//This is the session manager that is passed to SessionInterceptor
JwtCookieSession sessionManager() =>
    new JwtCookieSession(authenticationCtx, authorizationCtx);

///JWT Authentication context used by [JwtAuthHeaderSessionManager]
final authenticationCtx = new AuthenticationConf.SymmetricKey(
    'sdgdflgujsdgndsflkgjsdlnwertwert78676', 'jaguar');

///JWT Authorization context used by [JwtAuthHeaderSessionManager]
final authorizationCtx = new AuthorizationConf.SymmetricKey(
    'sdgdflgujsdgndsflkgjsdlnwertwert78676', 'jaguar');

class BaseApi {
  BasicAuth basicAuth(Context ctx) => new BasicAuth(kModelManager,
      authorizationIdKey: 'sub', sessionManager: sessionManager());

  Authorizer authorizer(Context ctx) => new Authorizer(kModelManager,
      authorizationIdKey: 'sub', sessionManager: sessionManager());
}

/// This route group contains login and logout authentication routes
@Api()
class AuthRoutes extends BaseApi {
  @Post(path: '/login')
  // Wrap [BasicAuth] interceptor to authenticate
  @WrapOne(#basicAuth)
  void login(Context ctx) {}

  @Post(path: '/logout')
  Future<Null> logout(Context ctx) async {
    final SessionManager sessionManager = ctx.getInput(Authorizer);
    await sessionManager.deleteSession();
  }
}

/// Collection of routes that need authorization
@Api(path: '/book')
// Wrap [Authorizer] interceptor to authorize requests
@WrapOne(#authorizer)
class StudentRoutes extends BaseApi {
  @Get(path: '/all')
  Response<String> getAllBooks(Context ctx) {
    // Authorized user can be obtained from Authorizer as Input
    User user = ctx.getInput(Authorizer);
    print(user.username);
    List<Map> ret = books.values.map((Book book) => book.toMap()).toList();
    return Response.json(ret);
  }

  @Get(path: '/:id')
  Response<String> getBook(Context ctx) {
    final String id = ctx.pathParams['id'];
    final Book book = books[id];
    return Response.json(book.toMap());
  }
}

@Api(path: '/api')
class LibraryApi {
  @IncludeApi()
  final AuthRoutes auth = new AuthRoutes();

  @IncludeApi()
  final StudentRoutes student = new StudentRoutes();
}

server() async {
  final api = new JaguarLibraryApi(new LibraryApi());

  Jaguar jaguar = new Jaguar();
  jaguar.addApi(api);

  await jaguar.serve();
}
